/*
**
**                           Main.c
**
**
**********************************************************************/
/*
   Last committed:     $Revision: 00 $
   Last changed by:    $Author: $
   Last changed date:  $Date:  $
   ID:                 $Id:  $

**********************************************************************/
#include "stm32f10x_conf.h"
#include "SysTimer.h"
#include "hd44780_driver.h"

extern uint32_t SystemCoreClock;  //    .

static void Init(void) //    .
{
    //    1,  A    .
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);

    //   /.
    GPIO_InitTypeDef GPIO;
    GPIO.GPIO_Pin = GPIO_Pin_8;       //   .
    GPIO.GPIO_Mode = GPIO_Mode_AF_PP; //     (   1).
    GPIO.GPIO_Speed = GPIO_Speed_50MHz;

    GPIO_Init(GPIOA, &GPIO);          //    A.

    GPIO.GPIO_Pin = GPIO_Pin_9;       //   .
    GPIO.GPIO_Mode = GPIO_Mode_IN_FLOATING; //  .
    GPIO_Init(GPIOA, &GPIO);          //    A.



    //   1.
    TIM_TimeBaseInitTypeDef Tim;
    TIM_TimeBaseStructInit(&Tim); //    .

    Tim.TIM_Prescaler = SystemCoreClock / 1000000 - 1; //    1000 .
    Tim.TIM_Period = 9999-10; //   0  9999   100   .

    TIM_TimeBaseInit(TIM1, &Tim);

    //    .
    TIM_OCInitTypeDef PWM; //   .
    TIM_OCStructInit(&PWM); //     .

    PWM.TIM_Pulse = 0;
    PWM.TIM_OCMode = TIM_OCMode_PWM2;
    PWM.TIM_OutputState = TIM_OutputState_Enable;
    PWM.TIM_OCPolarity = TIM_OCPolarity_Low;

    TIM_OC1Init(TIM1, &PWM); //  1   1.

    TIM_BDTRInitTypeDef bdtr;
    TIM_BDTRStructInit(&bdtr);
    bdtr.TIM_AutomaticOutput = TIM_AutomaticOutput_Enable;
    TIM_BDTRConfig(TIM1, &bdtr);

    TIM_SelectOnePulseMode(TIM1, TIM_OPMode_Single); //   OnePulse.

    // TIM_ARRPreloadConfig(TIM1, DISABLE);
    TIM_OC1PreloadConfig(TIM1, DISABLE); //    1 .



    //     .
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource9); //    9  .

    EXTI_InitTypeDef Exti;

    Exti.EXTI_Line = EXTI_Line9;
    Exti.EXTI_LineCmd = ENABLE;
    Exti.EXTI_Mode = EXTI_Mode_Interrupt;
    Exti.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_Init(&Exti);

    //  .
    NVIC_InitTypeDef Int;
    Int.NVIC_IRQChannel = EXTI9_5_IRQn;
    Int.NVIC_IRQChannelPreemptionPriority = 0; //   .
    Int.NVIC_IRQChannelSubPriority = 0;
    Int.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&Int);

}

static void InitButton(void)
{
    GPIO_InitTypeDef GPIO;

    GPIO.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11;  //   .
    GPIO.GPIO_Mode = GPIO_Mode_IN_FLOATING;     //  .
    GPIO_Init(GPIOA, &GPIO);
}

static void SetPowerPercent(uint16_t Percent)
{
    TIM1->CCR1 = TIM1->ARR - Percent;
}

static void LCD_Update(uint16_t Percent)
{
   static char Str[8]={'0', '0', '.', '0', '0', ' ', '%', 0};
   uint8_t i;

   lcd_set_xy(0,0);

   for (i=0; i<=4; i++)
   {
       if (i==2) i++;
       Str[4-i] = (Percent % 10) + 0x30;
       Percent /= 10;
   }

   lcd_out(Str);
}

int main(void)
{
    static uint16_t Percent=2000, OldPercent=0;
    static uint32_t Tick;
    static volatile uint32_t i;

    SystemCoreClockUpdate(); //   .
    SysTim_Init(1000);
    Init();                  //    .
    InitButton();

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);
    lcd_init();
    lcd_set_state(LCD_ENABLE, CURSOR_DISABLE, NO_BLINK);

    Tick = SysTim_GetTick();

    while(1)
    {
        SysTim_DelayMS(2);

        if ((GPIOA->IDR & GPIO_Pin_10) == 0) //  Up.
        {
            if (Percent < TIM1->ARR) Percent++;
        }

        if ((GPIOA->IDR & GPIO_Pin_11) == 0) //  Down.
        {
            if (Percent>0) Percent--;
        }


        if ((SysTim_GetTick()-Tick)>=100) //  100 .
        {
            Tick = SysTim_GetTick();

            if (Percent != OldPercent)
            {
                OldPercent = Percent;
                SetPowerPercent(Percent);
                LCD_Update(Percent);
            }
        }

    }
}


void EXTI9_5_IRQHandler(void) //      5 - 9.
{
    if (EXTI_GetITStatus(EXTI_Line9) != RESET) //    9.
    {
        EXTI_ClearITPendingBit(EXTI_Line9); //      9.
        TIM_Cmd(TIM1, ENABLE); //  .
    }
}
